@@ -1,4 +1,5 @@
-#   Copyright (C) 2006-2008 Ludovic Rousseau (ludovic.rousseau@free.fr)
+#   Copyright (C) 2006-2010 Ludovic Rousseau (ludovic.rousseau@free.fr)
+#   Copyright (C) 2010 Giuseppe Amato (additions to original interface)
 #
 # This file is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -14,6 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
+# $Id: __init__.py 227 2010-12-03 08:49:13Z lrousseau $
+
 import PyKCS11.LowLevel
 import os
 
@@ -56,7 +59,141 @@ CKR[-2] = "Unkown PKCS#11 type"
 CKR[-1] = "Load"
 
 
-class CK_SLOT_INFO(object):
+class ckbytelist(PyKCS11.LowLevel.ckbytelist):
+    """
+    add a __repr__() method to the LowLevel equivalent
+    """
+
+    def __repr__(self):
+        """
+        return the representation of a tuple
+        the __str__ method will use it also
+        """
+        rep = [elt for elt in self]
+        return repr(rep)
+
+
+class byteArray(PyKCS11.LowLevel.byteArray):
+    """
+    add a __repr__() method to the LowLevel equivalent
+    """
+
+    def __repr__(self):
+        """
+        return the representation of a tuple
+        the __str__ method will use it also
+        """
+        rep = [elt for elt in self]
+        return repr(rep)
+
+
+class CK_OBJECT_HANDLE(PyKCS11.LowLevel.CK_OBJECT_HANDLE):
+    """
+    add a __repr__() method to the LowLevel equivalent
+    """
+
+    def __init__(self, session):
+        PyKCS11.LowLevel.CK_OBJECT_HANDLE.__init__(self)
+        self.session = session
+        pass
+
+    def to_dict(self):
+        """
+        convert the fields of the object into a dictionnary
+        """
+        # all the attibutes defined by PKCS#11
+        all_attributes = PyKCS11.CKA.keys()
+
+        # only use the integer values and not the strings like 'CKM_RSA_PKCS'
+        all_attributes = [attr for attr in all_attributes if
+            isinstance(attr, int)]
+
+        # all the attributes of the object
+        attributes = self.session.getAttributeValue(self, all_attributes)
+
+        dico = dict()
+        for key, attr in zip(all_attributes, attributes):
+            if attr == None:
+                continue
+            if key == CKA_CLASS:
+                dico[PyKCS11.CKA[key]] = PyKCS11.CKO[attr]
+            elif key == CKA_CERTIFICATE_TYPE:
+                dico[PyKCS11.CKA[key]] = PyKCS11.CKC[attr]
+            elif key == CKA_KEY_TYPE:
+                dico[PyKCS11.CKA[key]] = PyKCS11.CKK[attr]
+            else:
+                dico[PyKCS11.CKA[key]] = attr
+        return dico
+
+    def __repr__(self):
+        """
+        text representation of the object
+        """
+        dico = self.to_dict()
+        lines = list()
+        for key in sorted(dico.keys()):
+            lines.append("%s: %s" % (key, dico[key]))
+        return "\n".join(lines)
+
+
+class CkClass(object):
+    """
+    Base class for CK_* classes
+    """
+
+    # dictionnary of integer_value: text_value for the flags bits
+    flags_dict = dict()
+
+    # dictionnary of fields names and types
+    # type can be "pair", "flags" or "text"
+    fields = dict()
+
+    flags = 0
+
+    def flags2text(self):
+        """
+        parse the L{self.flags} field and create a list of "CKF_*" strings
+        corresponding to bits set in flags
+
+        @return: a list of strings
+        @rtype: list
+        """
+        r = []
+        for v in self.flags_dict.keys():
+            if self.flags & v:
+                r.append(self.flags_dict[v])
+        return r
+
+    def to_dict(self):
+        """
+        convert the fields of the object into a dictionnary
+        """
+        dico = dict()
+        for field in self.fields.keys():
+            if field == "flags":
+                dico[field] = self.flags2text()
+            else:
+                dico[field] = eval("self." + field)
+        return dico
+
+    def __str__(self):
+        """
+        text representation of the object
+        """
+        dico = self.to_dict()
+        lines = list()
+        for key in sorted(dico.keys()):
+            type = self.fields[key]
+            if type == "flags":
+                lines.append("%s: %s" % (key, ", ".join(dico[key])))
+            elif type == "pair":
+                lines.append("%s: " % key + "%d.%d" % dico[key])
+            else:
+                lines.append("%s: %s" % (key, dico[key]))
+        return "\n".join(lines)
+
+
+class CK_SLOT_INFO(CkClass):
     """
     matches the PKCS#11 CK_SLOT_INFO structure
 
@@ -77,22 +214,14 @@ class CK_SLOT_INFO(object):
         CKF_REMOVABLE_DEVICE: "CKF_REMOVABLE_DEVICE",
         CKF_HW_SLOT: "CKF_HW_SLOT"}
 
-    def flags2text(self):
-        """
-        parse the L{self.flags} field and create a list of "CKF_*" strings
-        corresponding to bits set in flags
-
-        @return: a list of strings
-        @rtype: list
-        """
-        r = []
-        for v in CK_SLOT_INFO.flags_dict.keys():
-            if self.flags & v:
-                r.append(CK_SLOT_INFO.flags_dict[v])
-        return r
+    fields = {"slotDescription": "text",
+        "manufacturerID": "text",
+        "flags": "flags",
+        "hardwareVersion": "text",
+        "firmwareVersion": "text"}
 
 
-class CK_INFO(object):
+class CK_INFO(CkClass):
     """
     matches the PKCS#11 CK_INFO structure
 
@@ -108,8 +237,14 @@ class CK_INFO(object):
     @type libraryVersion: list
     """
 
+    fields = {"cryptokiVersion": "pair",
+        "manufacturerID": "text",
+        "flags": "flags",
+        "libraryDescription": "text",
+        "libraryVersion": "pair"}
 
-class CK_SESSION_INFO(object):
+
+class CK_SESSION_INFO(CkClass):
     """
     matches the PKCS#11 CK_SESSION_INFO structure
 
@@ -128,20 +263,6 @@ class CK_SESSION_INFO(object):
         CKF_SERIAL_SESSION: "CKF_SERIAL_SESSION",
     }
 
-    def flags2text(self):
-        """
-        parse the L{self.flags} field and create a list of "CKF_*" strings
-        corresponding to bits set in flags
-
-        @return: a list of strings
-        @rtype: list
-        """
-        r = []
-        for v in CK_SESSION_INFO.flags_dict.keys():
-            if self.flags & v:
-                r.append(CK_SESSION_INFO.flags_dict[v])
-        return r
-
     def state2text(self):
         """
         parse the L{self.state} field and return a "CKS_*" string
@@ -152,8 +273,13 @@ class CK_SESSION_INFO(object):
         """
         return CKS[self.state]
 
+    fields = {"slotID": "text",
+        "state": "text",
+        "flags": "flags",
+        "ulDeviceError": "text"}
 
-class CK_TOKEN_INFO(object):
+
+class CK_TOKEN_INFO(CkClass):
     """
     matches the PKCS#11 CK_TOKEN_INFO structure
 
@@ -216,22 +342,27 @@ class CK_TOKEN_INFO(object):
         CKF_SO_PIN_TO_BE_CHANGED: "CKF_SO_PIN_TO_BE_CHANGED",
     }
 
-    def flags2text(self):
-        """
-        parse the L{flags} field and create a list of "CKF_*" strings
-        corresponding to bits set in flags
-
-        @return: a list of strings
-        @rtype: list
-        """
-        r = []
-        for v in CK_TOKEN_INFO.flags_dict.keys():
-            if self.flags & v:
-                r.append(CK_TOKEN_INFO.flags_dict[v])
-        return r
+    fields = {"label": "text",
+        "manufacturerID": "text",
+        "model": "text",
+        "serialNumber": "text",
+        "flags": "flags",
+        "ulMaxSessionCount": "text",
+        "ulSessionCount": "text",
+        "ulMaxRwSessionCount": "text",
+        "ulRwSessionCount": "text",
+        "ulMaxPinLen": "text",
+        "ulMinPinLen": "text",
+        "ulTotalPublicMemory": "text",
+        "ulFreePublicMemory": "text",
+        "ulTotalPrivateMemory": "text",
+        "ulFreePrivateMemory": "text",
+        "hardwareVersion": "pair",
+        "firmwareVersion": "pair",
+        "utcTime": "text"}
 
 
-class CK_MECHANISM_INFO(object):
+class CK_MECHANISM_INFO(CkClass):
     """
     matches the PKCS#11 CK_MECHANISM_INFO structure
 
@@ -260,19 +391,9 @@ class CK_MECHANISM_INFO(object):
         CKF_EXTENSION: "CKF_EXTENSION",
     }
 
-    def flags2text(self):
-        """
-        parse the L{self.flags} field and create a list of "CKF_*" strings
-        corresponding to bits set in flags
-
-        @return: a list of strings
-        @rtype: list
-        """
-        r = []
-        for v in CK_MECHANISM_INFO.flags_dict.keys():
-            if self.flags & v:
-                r.append(CK_MECHANISM_INFO.flags_dict[v])
-        return r
+    fields = {"ulMinKeySize": "text",
+        "ulMaxKeySize": "text",
+        "flags": "flags"}
 
 
 class PyKCS11Error(Exception):
@@ -320,6 +441,14 @@ class PyKCS11Lib(object):
         if rv == 0:
             raise PyKCS11Error(-1, pkcs11dll_filename)
 
+    def initToken(self, slot, pin, label):
+        """
+        C_InitToken
+        """
+        rv = self.lib.C_InitToken(slot, pin, label)
+        if rv != CKR_OK:
+            raise PyKCS11Error(rv)
+
     def getInfo(self):
         """
         C_GetInfo
@@ -412,8 +541,6 @@ class PyKCS11Lib(object):
         t.ulMaxPinLen = tokeninfo.ulMaxPinLen
         t.ulMinPinLen = tokeninfo.ulMinPinLen
         t.ulTotalPublicMemory = tokeninfo.ulTotalPublicMemory
-        CKU_SO = PyKCS11.LowLevel.CKU_SO
-        CKU_USER = PyKCS11.LowLevel.CKU_USER
 
         if t.ulTotalPublicMemory == CK_UNAVAILABLE_INFORMATION:
             t.ulTotalPublicMemory = -1
@@ -524,6 +651,7 @@ class Mechanism(object):
         self.param = param
 
 MechanismRSAPKCS1 = Mechanism(CKM_RSA_PKCS, None)
+MechanismRSAGENERATEKEYPAIR = Mechanism(CKM_RSA_PKCS_KEY_PAIR_GEN, None)
 
 
 class Session(object):
@@ -587,14 +715,6 @@ class Session(object):
 
         del self
 
-    def initToken(self, pin, label):
-        """
-        C_InitToken
-        """
-        rv = self.lib.C_InitToken(self.session, pin, label)
-        if rv != CKR_OK:
-            raise PyKCS11Error(rv)
-
     def initPin(self, new_pin):
         """
         C_InitPIN
@@ -611,6 +731,11 @@ class Session(object):
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
 
+    def destroyObject(self, obj):
+        rv = self.lib.C_DestroyObject(self.session, obj)
+        if rv != CKR_OK:
+            raise PyKCS11Error(rv)
+
     def sign(self, key, data, mecha=MechanismRSAPKCS1):
         """
         C_SignInit/C_Sign
@@ -625,13 +750,13 @@ class Session(object):
         @return: the computed signature
         @rtype: list of bytes
 
-        @note: the returned value is an istance of L{LowLevel.ckbytelist}.
+        @note: the returned value is an istance of L{ckbytelist}.
         You can easly convert it to a binary string with::
             ''.join(chr(i) for i in ckbytelistSignature)
 
         """
         m = PyKCS11.LowLevel.CK_MECHANISM()
-        signature = PyKCS11.LowLevel.ckbytelist()
+        signature = ckbytelist()
         ba = None  # must be declared here or may be deallocated too early
         m.mechanism = mecha.mechanism
         if (mecha.param):
@@ -647,7 +772,7 @@ class Session(object):
             # pParameter is an opaque pointer, never garbage collected.
             m.pParameter = ba.cast()
             m.ulParameterLen = len(mecha.param)
-        data1 = PyKCS11.LowLevel.ckbytelist()
+        data1 = ckbytelist()
         data1.reserve(len(data))
         if type(data) is type(''):
             for x in data:
@@ -656,15 +781,15 @@ class Session(object):
             for c in xrange(len(data)):
                 data1.append(data[c])
         rv = self.lib.C_SignInit(self.session, m, key)
-        if (rv != 0):
+        if rv != CKR_OK:
             raise PyKCS11Error(rv)
         #first call get signature size
         rv = self.lib.C_Sign(self.session, data1, signature)
-        if (rv != 0):
+        if rv != CKR_OK:
             raise PyKCS11Error(rv)
         #second call get actual signature data
         rv = self.lib.C_Sign(self.session, data1, signature)
-        if (rv != 0):
+        if rv != CKR_OK:
             raise PyKCS11Error(rv)
         return signature
 
@@ -682,13 +807,13 @@ class Session(object):
         @return: the decrypted data
         @rtype: list of bytes
 
-        @note: the returned value is an istance of L{LowLevel.ckbytelist}.
+        @note: the returned value is an istance of L{ckbytelist}.
         You can easly convert it to a binary string with::
             ''.join(chr(i) for i in ckbytelistData)
 
         """
         m = PyKCS11.LowLevel.CK_MECHANISM()
-        decrypted = PyKCS11.LowLevel.ckbytelist()
+        decrypted = ckbytelist()
         ba = None  # must be declared here or may be deallocated too early
         m.mechanism = mecha.mechanism
         if (mecha.param):
@@ -704,7 +829,7 @@ class Session(object):
             # pParameter is an opaque pointer, never garbage collected.
             m.pParameter = ba.cast()
             m.ulParameterLen = len(mecha.param)
-        data1 = PyKCS11.LowLevel.ckbytelist()
+        data1 = ckbytelist()
         data1.reserve(len(data))
         if type(data) is type(''):
             for x in data:
@@ -713,15 +838,15 @@ class Session(object):
             for c in xrange(len(data)):
                 data1.append(data[c])
         rv = self.lib.C_DecryptInit(self.session, m, key)
-        if (rv != 0):
+        if rv != CKR_OK:
             raise PyKCS11Error(rv)
         #first call get decrypted size
         rv = self.lib.C_Decrypt(self.session, data1, decrypted)
-        if (rv != 0):
+        if rv != CKR_OK:
             raise PyKCS11Error(rv)
         #second call get actual decrypted data
         rv = self.lib.C_Decrypt(self.session, data1, decrypted)
-        if (rv != 0):
+        if rv != CKR_OK:
             raise PyKCS11Error(rv)
         return decrypted
 
@@ -768,6 +893,58 @@ class Session(object):
     def isBin(self, type):
         return (not self.isBool(type)) and (not self.isString(type)) and (not self.isNum(type))
 
+    def _template2ckattrlist(self, template):
+        t = PyKCS11.LowLevel.ckattrlist(len(template))
+        for x in xrange(len(template)):
+            attr = template[x]
+            if self.isNum(attr[0]):
+                t[x].SetNum(attr[0], int(attr[1]))
+            elif self.isString(attr[0]):
+                t[x].SetString(attr[0], str(attr[1]))
+            elif self.isBool(attr[0]):
+                t[x].SetBool(attr[0], attr[1])
+            elif self.isBin(attr[0]):
+                attrBin = attr[1]
+                attrStr = attr[1]
+                if isinstance(attr[1], int):
+                    attrStr = str(attr[1])
+                if isinstance(attr[1], str):
+                    attrBin = ckbytelist(len(attrStr))
+                    for c in xrange(len(attrStr)):
+                        attrBin.append(ord(attrStr[c]))
+                t[x].SetBin(attr[0], attrBin)
+            else:
+                raise PyKCS11Error(-2)
+        return t
+
+    def generateKeyPair(self, templatePub, templatePriv, mecha=MechanismRSAGENERATEKEYPAIR):
+        tPub = self._template2ckattrlist(templatePub)
+        tPriv = self._template2ckattrlist(templatePriv)
+        ck_pub_handle = PyKCS11.LowLevel.CK_OBJECT_HANDLE()
+        ck_prv_handle = PyKCS11.LowLevel.CK_OBJECT_HANDLE()
+        m = PyKCS11.LowLevel.CK_MECHANISM()
+        ba = None # must be declared here or may be deallocated too early
+        m.mechanism = mecha.mechanism
+        if (mecha.param):
+            ba = PyKCS11.LowLevel.byteArray(len(mecha.param))
+            if type(mecha.param) is type(''):
+                for c in xrange(len(mecha.param)):
+                    ba[c] = ord(mecha.param[c])
+            else:
+                for c in xrange(len(mecha.param)):
+                    ba[c] = mecha.param[c]
+            # with cast() the ba object continue to own internal pointer
+            # (avoids a leak).
+            # pParameter is an opaque pointer, never garbage collected.
+            m.pParameter = ba.cast()
+            m.ulParameterLen = len(mecha.param)
+        rv = self.lib.C_GenerateKeyPair(self.session, m, tPub, tPriv,
+            ck_pub_handle, ck_prv_handle)
+
+        if rv != CKR_OK:
+            raise PyKCS11Error(rv)
+        return ck_pub_handle, ck_prv_handle
+
     def findObjects(self, template=()):
         """
         find the objects matching the template pattern
@@ -778,19 +955,7 @@ class Session(object):
         @return: a list of object ids
         @rtype: list
         """
-        t = PyKCS11.LowLevel.ckattrlist(len(template))
-        for x in xrange(len(template)):
-            attr = template[x]
-            if self.isNum(attr[0]):
-                t[x].SetNum(attr[0], attr[1])
-            elif self.isString(attr[0]):
-                t[x].SetString(attr[0], attr[1])
-            elif self.isBool(attr[0]):
-                t[x].SetBool(attr[0], attr[1])
-            elif self.isBin(attr[0]):
-                t[x].SetBin(attr[0], attr[1])
-            else:
-                raise PyKCS11Error(-2)
+        t = self._template2ckattrlist(template)
 
         # we search for 10 objects by default. speed/memory tradeoff
         result = PyKCS11.LowLevel.ckobjlist(10)
@@ -803,7 +968,7 @@ class Session(object):
             for x in result:
                 # make a copy of the handle: the original value get
                 # corrupted (!!)
-                a = PyKCS11.LowLevel.CK_OBJECT_HANDLE()
+                a = CK_OBJECT_HANDLE(self)
                 a.assign(x.value())
                 res.append(a)
             if len(result) == 0:
@@ -829,7 +994,7 @@ class Session(object):
 
         @note: if allAsBinary is True the function do not convert results to
         Python types (i.e.: CKA_TOKEN to Bool, CKA_CLASS to int, ...).
-        Binary data is returned as L{LowLevel.ckbytelist} type, usable
+        Binary data is returned as L{ckbytelist} type, usable
         as a list containing only bytes.
         You can easly convert it to a binary string with::
             ''.join(chr(i) for i in ckbytelistVariable)
@@ -871,9 +1036,9 @@ class Session(object):
     def getAttributeValue_fragmented(self, obj_id, attr, allAsBinary=False):
         """
         Same as L{getAttributeValue} except that when some attribute
-        is sensitive or unknown an empty value (None) is retruned.
+        is sensitive or unknown an empty value (None) is returned.
 
-        Note: this is achived getting attributes one by one.
+        Note: this is achived by getting attributes one by one.
 
         @see: L{getAttributeValue}
         """
@@ -921,7 +1086,7 @@ class Session(object):
         @param seed: seed material
         @type seed: iterable
         """
-        low_seed = PyKCS11.LowLevel.ckbytelist(len(seed))
+        low_seed = ckbytelist(len(seed))
         for c in xrange(len(seed)):
             low_seed.append(seed[c])
         rv = self.lib.C_SeedRandom(self.session, low_seed)
@@ -935,11 +1100,11 @@ class Session(object):
         @param size: number of random bytes to get
         @type size: integer
 
-        @note: the returned value is an istance of L{LowLevel.ckbytelist}.
+        @note: the returned value is an istance of L{ckbytelist}.
         You can easly convert it to a binary string with::
-            ''.join(chr(i) for i in ckbytelistSignature)
+            ''.join(chr(i) for i in random)
         """
-        low_rand = PyKCS11.LowLevel.ckbytelist(size)
+        low_rand = ckbytelist(size)
         rv = self.lib.C_GenerateRandom(self.session, low_rand)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -951,52 +1116,22 @@ if __name__ == "__main__":
     p.load()
 
     print "getInfo"
-    i = p.getInfo()
-    print "cryptokiVersion: %d.%d" % i.cryptokiVersion
-    print "manufacturerID:", i.manufacturerID
-    print "flags:", i.flags
-    print "libraryDescription:", i.libraryDescription
-    print "libraryVersion: %d.%d" % i.libraryVersion
+    print p.getInfo()
 
     print
     print "getSlotList"
     s = p.getSlotList()
     print "slots:", s
-    slot = s[1]
+    slot = s[0]
     print "using slot:", slot
 
     print
     print "getSlotInfo"
-    i = p.getSlotInfo(slot)
-    print "slotDescription:", i.slotDescription.strip()
-    print "manufacturerID:", i.manufacturerID
-    print "flags:", i.flags
-    print "flags:", i.flags2text()
-    print "hardwareVersion:", i.hardwareVersion
-    print "firmwareVersion:", i.firmwareVersion
+    print p.getSlotInfo(slot)
 
     print
     print "getTokenInfo"
-    t = p.getTokenInfo(slot)
-    print "label:", t.label
-    print "manufacturerID:", t.manufacturerID
-    print "model:", t.model
-    print "serialNumber:", t.serialNumber
-    print "flags:", t.flags
-    print "flags:", t.flags2text()
-    print "ulMaxSessionCount:", t.ulMaxSessionCount
-    print "ulSessionCount:", t.ulSessionCount
-    print "ulMaxRwSessionCount:", t.ulMaxRwSessionCount
-    print "ulRwSessionCount:", t.ulRwSessionCount
-    print "ulMaxPinLen:", t.ulMaxPinLen
-    print "ulMinPinLen:", t.ulMinPinLen
-    print "ulTotalPublicMemory:", t.ulTotalPublicMemory
-    print "ulFreePublicMemory:", t.ulFreePublicMemory
-    print "ulTotalPrivateMemory:", t.ulTotalPrivateMemory
-    print "ulFreePrivateMemory:", t.ulFreePrivateMemory
-    print "hardwareVersion: %d.%d" % t.hardwareVersion
-    print "firmwareVersion: %d.%d" % t.firmwareVersion
-    print "utcTime:", t.utcTime
+    print p.getTokenInfo(slot)
 
     print
     print "openSession"
@@ -1004,13 +1139,7 @@ if __name__ == "__main__":
 
     print
     print "sessionInfo"
-    si = se.getSessionInfo()
-    print "slotID:", si.slotID
-    print "state:", si.state
-    print "state:", si.state2text()
-    print "flags:", si.flags
-    print "flags:", si.flags2text()
-    print "ulDeviceError:", si.ulDeviceError
+    print se.getSessionInfo()
 
     print
     print "seedRandom"
@@ -1019,26 +1148,20 @@ if __name__ == "__main__":
     except PyKCS11Error, e:
         print e
     print "generateRandom"
-    rand = se.generateRandom()
-    print " ".join("%02X" % i for i in rand)
+    print se.generateRandom()
 
     print
     print "login"
-    se.login(pin="12345678")
+    se.login(pin="0000")
 
     print
     print "sessionInfo"
-    si = se.getSessionInfo()
-    print "slotID:", si.slotID
-    print "state:", si.state
-    print "state:", si.state2text()
-    print "flags:", si.flags
-    print "flags:", si.flags2text()
-    print "ulDeviceError:", si.ulDeviceError
+    print se.getSessionInfo()
 
     print
     print "findObjects"
     objs = se.findObjects([(CKA_CLASS, CKO_CERTIFICATE)])
+    print "Nb objetcs:", len(objs)
     print objs
 
     print
